using System;
using System.Collections.Generic;
using System.Text;
using System.Net.Sockets;
using System.IO;
using System.Globalization;

namespace LNZClient_CS
{
    class Program
    {
        static void event_loop(TcpClient cli, StreamReader reader, StreamWriter writer)
        {
            try
            {
                Lovec t;
                string cmd;
                string[] cmda;
                char[] delimiter = new char[] { ' ' };

                while (true)
                {
                    cmd = reader.ReadLine();
                    cmda = cmd.Split(delimiter);

                    if (cmda[0] == "ADIJO") return;
                    if (cmda[0].StartsWith("KONEC_IGRE")) continue;
                    if (cmda[0] != "ZBUDI_SE") throw new Exception("Napaka pri komunikaciji s streznikom.");

                    int[] cmdi = new int[cmda.Length - 1];
                    for (int i = 0; i < cmdi.Length; i++)
                    {
                        cmdi[i] = Int32.Parse(cmda[i + 1], CultureInfo.InvariantCulture);
                    }

                    int N = cmdi[0];
                    int K = cmdi[1];

                    Koordinate j0 = new Koordinate(cmdi[2], cmdi[3]);
                    Koordinate n0 = new Koordinate(cmdi[4], cmdi[5]);

                    Koordinate[] zakladi = new Koordinate[K];
                    for (int i = 0; i < K; i++) zakladi[i] = new Koordinate(cmdi[6 + 2 * i], cmdi[6 + 2 * i + 1]);

                    t = new Lovec();
                    t.ZbudiSe(N, K, j0, n0, zakladi);

                    writer.WriteLine("BUDEN");

                    while (true)
                    {
                        cmd = reader.ReadLine();
                        cmda = cmd.Split(delimiter);

                        if (cmda[0] == "ADIJO") return;
                        if (cmda[0] == "KONEC_IGRE")
                        {
                            int razlog = Int32.Parse(cmda[1]);
                            t.KonecIgre();
                            t = null;
                            break;
                        }
                        else if (cmda[0] == "QV")
                        {
                            Koordinate j = new Koordinate(Int32.Parse(cmda[1]), Int32.Parse(cmda[2]));
                            Koordinate n = new Koordinate(Int32.Parse(cmda[3]), Int32.Parse(cmda[4]));
                            zakladi = pojej(zakladi, j);
                            zakladi = pojej(zakladi, n);
                            int dx = 0, dy = 0;
                            t.KamGres(j, n, zakladi, out dx, out dy);
                            writer.WriteLine("QVR " + dx.ToString() + " " + dy.ToString());
                            //                            Console.WriteLine(String.Format(CultureInfo.InvariantCulture, 
                            //                                    "d={0:F5} dD={1:F5} v={2:F5} dV={3:F}", j.smer.d, SpremembaSmeri, j.smer.v, SpremembaHitrosti));
                        }
                        else
                        {
                        }
                    }
                }
            }
            catch (Exception e)
            {
                Console.WriteLine("Napaka: " + e.ToString());
            }
        }

        static Koordinate[] pojej(Koordinate[] zakladi, Koordinate k)
        {
            int z = 0;
            for (int i = 0; i < zakladi.Length; i++)
            {
                if (!((zakladi[i].x == k.x) && (zakladi[i].y == k.y))) ++z;
            }
            Koordinate[] rz = new Koordinate[z];
            int t = 0;
            for (int i = 0; i < zakladi.Length; i++)
            {
                if (!((zakladi[i].x == k.x) && (zakladi[i].y == k.y)))
                {
                    rz[t++] = zakladi[i];
                }
            }
            return rz;
        }

        static TcpClient connect(string hostname, out StreamReader reader, out StreamWriter writer)
        {
            TcpClient client = new TcpClient(hostname, 1175);
            reader = new StreamReader(client.GetStream());
            writer = new StreamWriter(client.GetStream());
            writer.AutoFlush = true;

            string server = reader.ReadLine();
            Console.WriteLine(server);

            writer.WriteLine("AVE " + Lovec.Ime);
            string succ = reader.ReadLine();
            if (succ != "PRIJAVA USPESNA") throw new Exception("Prijava na streznik neuspesna: " + succ);
            return client;
        }

        static void Main(string[] args)
        {
            string hostname;
            if (args.Length < 2) hostname = "localhost";
            else hostname = args[1];

            StreamReader reader;
            StreamWriter writer;
            TcpClient client = connect(hostname, out reader, out writer);
            event_loop(client, reader, writer);
        }
    }

    /// <summary>
    /// Pozicija zivali
    /// </summary>
    class Koordinate
    {
        public int x, y;
        public Koordinate(int _x, int _y)
        {
            x = _x;
            y = _y;
        }
    }
}
